# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmModelServiceGatewayTrafficSplitEntry(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'port': 'int',
        'revision_number': 'int',
        'traffic_weight': 'int'
    }

    attribute_map = {
        'port': 'port',
        'revision_number': 'revision_number',
        'traffic_weight': 'traffic_weight'
    }

    def __init__(self, port=None, revision_number=None, traffic_weight=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelServiceGatewayTrafficSplitEntry - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._port = None
        self._revision_number = None
        self._traffic_weight = None
        self.discriminator = None

        if port is not None:
            self.port = port
        if revision_number is not None:
            self.revision_number = revision_number
        if traffic_weight is not None:
            self.traffic_weight = traffic_weight

    @property
    def port(self):
        """Gets the port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._port

    @port.setter
    def port(self, port):
        """Sets the port of this OrmModelServiceGatewayTrafficSplitEntry.


        :param port: The port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type port: int
        """
        if (self.local_vars_configuration.client_side_validation and
                port is not None and port > 65535):  # noqa: E501
            raise ValueError("Invalid value for `port`, must be a value less than or equal to `65535`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                port is not None and port < 1):  # noqa: E501
            raise ValueError("Invalid value for `port`, must be a value greater than or equal to `1`")  # noqa: E501

        self._port = port

    @property
    def revision_number(self):
        """Gets the revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._revision_number

    @revision_number.setter
    def revision_number(self, revision_number):
        """Sets the revision_number of this OrmModelServiceGatewayTrafficSplitEntry.


        :param revision_number: The revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type revision_number: int
        """

        self._revision_number = revision_number

    @property
    def traffic_weight(self):
        """Gets the traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._traffic_weight

    @traffic_weight.setter
    def traffic_weight(self, traffic_weight):
        """Sets the traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.


        :param traffic_weight: The traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type traffic_weight: int
        """

        self._traffic_weight = traffic_weight

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelServiceGatewayTrafficSplitEntry):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelServiceGatewayTrafficSplitEntry):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ClusterClusterWorkload(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'experiment': 'ResponseExperimentListResponse',
        'id': 'int',
        'kernel_cluster_node': 'ResponseKernelClusterNode',
        'kernel_resource_spec': 'ResponseKernelResourceSpec',
        'model_service_revision': 'ResponseModelServiceRevision',
        'pipeline_step_execution': 'ResponsePipelineStepExecution',
        'status': 'str',
        'status_last_updated': 'datetime',
        'system_metrics': 'list[InfluxdbSystemMetricList]',
        'type': 'str',
        'workspace': 'ResponseWorkspaceList'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'experiment': 'experiment',
        'id': 'id',
        'kernel_cluster_node': 'kernel_cluster_node',
        'kernel_resource_spec': 'kernel_resource_spec',
        'model_service_revision': 'model_service_revision',
        'pipeline_step_execution': 'pipeline_step_execution',
        'status': 'status',
        'status_last_updated': 'status_last_updated',
        'system_metrics': 'system_metrics',
        'type': 'type',
        'workspace': 'workspace'
    }

    def __init__(self, created_dt=None, experiment=None, id=None, kernel_cluster_node=None, kernel_resource_spec=None, model_service_revision=None, pipeline_step_execution=None, status=None, status_last_updated=None, system_metrics=None, type=None, workspace=None, local_vars_configuration=None):  # noqa: E501
        """ClusterClusterWorkload - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._experiment = None
        self._id = None
        self._kernel_cluster_node = None
        self._kernel_resource_spec = None
        self._model_service_revision = None
        self._pipeline_step_execution = None
        self._status = None
        self._status_last_updated = None
        self._system_metrics = None
        self._type = None
        self._workspace = None
        self.discriminator = None

        self.created_dt = created_dt
        if experiment is not None:
            self.experiment = experiment
        self.id = id
        if kernel_cluster_node is not None:
            self.kernel_cluster_node = kernel_cluster_node
        if kernel_resource_spec is not None:
            self.kernel_resource_spec = kernel_resource_spec
        if model_service_revision is not None:
            self.model_service_revision = model_service_revision
        if pipeline_step_execution is not None:
            self.pipeline_step_execution = pipeline_step_execution
        self.status = status
        self.status_last_updated = status_last_updated
        if system_metrics is not None:
            self.system_metrics = system_metrics
        self.type = type
        if workspace is not None:
            self.workspace = workspace

    @property
    def created_dt(self):
        """Gets the created_dt of this ClusterClusterWorkload.  # noqa: E501


        :return: The created_dt of this ClusterClusterWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ClusterClusterWorkload.


        :param created_dt: The created_dt of this ClusterClusterWorkload.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def experiment(self):
        """Gets the experiment of this ClusterClusterWorkload.  # noqa: E501


        :return: The experiment of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponseExperimentListResponse
        """
        return self._experiment

    @experiment.setter
    def experiment(self, experiment):
        """Sets the experiment of this ClusterClusterWorkload.


        :param experiment: The experiment of this ClusterClusterWorkload.  # noqa: E501
        :type experiment: ResponseExperimentListResponse
        """

        self._experiment = experiment

    @property
    def id(self):
        """Gets the id of this ClusterClusterWorkload.  # noqa: E501


        :return: The id of this ClusterClusterWorkload.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ClusterClusterWorkload.


        :param id: The id of this ClusterClusterWorkload.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def kernel_cluster_node(self):
        """Gets the kernel_cluster_node of this ClusterClusterWorkload.  # noqa: E501


        :return: The kernel_cluster_node of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponseKernelClusterNode
        """
        return self._kernel_cluster_node

    @kernel_cluster_node.setter
    def kernel_cluster_node(self, kernel_cluster_node):
        """Sets the kernel_cluster_node of this ClusterClusterWorkload.


        :param kernel_cluster_node: The kernel_cluster_node of this ClusterClusterWorkload.  # noqa: E501
        :type kernel_cluster_node: ResponseKernelClusterNode
        """

        self._kernel_cluster_node = kernel_cluster_node

    @property
    def kernel_resource_spec(self):
        """Gets the kernel_resource_spec of this ClusterClusterWorkload.  # noqa: E501


        :return: The kernel_resource_spec of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponseKernelResourceSpec
        """
        return self._kernel_resource_spec

    @kernel_resource_spec.setter
    def kernel_resource_spec(self, kernel_resource_spec):
        """Sets the kernel_resource_spec of this ClusterClusterWorkload.


        :param kernel_resource_spec: The kernel_resource_spec of this ClusterClusterWorkload.  # noqa: E501
        :type kernel_resource_spec: ResponseKernelResourceSpec
        """

        self._kernel_resource_spec = kernel_resource_spec

    @property
    def model_service_revision(self):
        """Gets the model_service_revision of this ClusterClusterWorkload.  # noqa: E501


        :return: The model_service_revision of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponseModelServiceRevision
        """
        return self._model_service_revision

    @model_service_revision.setter
    def model_service_revision(self, model_service_revision):
        """Sets the model_service_revision of this ClusterClusterWorkload.


        :param model_service_revision: The model_service_revision of this ClusterClusterWorkload.  # noqa: E501
        :type model_service_revision: ResponseModelServiceRevision
        """

        self._model_service_revision = model_service_revision

    @property
    def pipeline_step_execution(self):
        """Gets the pipeline_step_execution of this ClusterClusterWorkload.  # noqa: E501


        :return: The pipeline_step_execution of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponsePipelineStepExecution
        """
        return self._pipeline_step_execution

    @pipeline_step_execution.setter
    def pipeline_step_execution(self, pipeline_step_execution):
        """Sets the pipeline_step_execution of this ClusterClusterWorkload.


        :param pipeline_step_execution: The pipeline_step_execution of this ClusterClusterWorkload.  # noqa: E501
        :type pipeline_step_execution: ResponsePipelineStepExecution
        """

        self._pipeline_step_execution = pipeline_step_execution

    @property
    def status(self):
        """Gets the status of this ClusterClusterWorkload.  # noqa: E501


        :return: The status of this ClusterClusterWorkload.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ClusterClusterWorkload.


        :param status: The status of this ClusterClusterWorkload.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def status_last_updated(self):
        """Gets the status_last_updated of this ClusterClusterWorkload.  # noqa: E501


        :return: The status_last_updated of this ClusterClusterWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._status_last_updated

    @status_last_updated.setter
    def status_last_updated(self, status_last_updated):
        """Sets the status_last_updated of this ClusterClusterWorkload.


        :param status_last_updated: The status_last_updated of this ClusterClusterWorkload.  # noqa: E501
        :type status_last_updated: datetime
        """
        if self.local_vars_configuration.client_side_validation and status_last_updated is None:  # noqa: E501
            raise ValueError("Invalid value for `status_last_updated`, must not be `None`")  # noqa: E501

        self._status_last_updated = status_last_updated

    @property
    def system_metrics(self):
        """Gets the system_metrics of this ClusterClusterWorkload.  # noqa: E501


        :return: The system_metrics of this ClusterClusterWorkload.  # noqa: E501
        :rtype: list[InfluxdbSystemMetricList]
        """
        return self._system_metrics

    @system_metrics.setter
    def system_metrics(self, system_metrics):
        """Sets the system_metrics of this ClusterClusterWorkload.


        :param system_metrics: The system_metrics of this ClusterClusterWorkload.  # noqa: E501
        :type system_metrics: list[InfluxdbSystemMetricList]
        """

        self._system_metrics = system_metrics

    @property
    def type(self):
        """Gets the type of this ClusterClusterWorkload.  # noqa: E501


        :return: The type of this ClusterClusterWorkload.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ClusterClusterWorkload.


        :param type: The type of this ClusterClusterWorkload.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def workspace(self):
        """Gets the workspace of this ClusterClusterWorkload.  # noqa: E501


        :return: The workspace of this ClusterClusterWorkload.  # noqa: E501
        :rtype: ResponseWorkspaceList
        """
        return self._workspace

    @workspace.setter
    def workspace(self, workspace):
        """Sets the workspace of this ClusterClusterWorkload.


        :param workspace: The workspace of this ClusterClusterWorkload.  # noqa: E501
        :type workspace: ResponseWorkspaceList
        """

        self._workspace = workspace

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterClusterWorkload):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterClusterWorkload):
            return True

        return self.to_dict() != other.to_dict()



from datetime import datetime
from typing import List

import click

from openapi_client import ResponseSimpleModelInfo
from vessl import list_experiment_output_files, list_experiments
from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    Endpoint,
    format_size,
    format_url,
    generic_prompter,
    print_data,
    print_info,
    print_success,
    print_table,
    print_volume_files,
    prompt_checkbox,
    prompt_choices,
    truncate_datetime,
)
from vessl.cli.dataset import download_dest_path_prompter
from vessl.cli.organization import organization_name_option
from vessl.cli.project import project_name_option
from vessl.experiment import read_experiment_by_id
from vessl.model import (
    create_model,
    create_model_repository,
    delete_model_volume_file,
    download_model_volume_file,
    list_model_repositories,
    list_model_volume_files,
    list_models,
    read_model,
    read_model_repository,
    upload_model_volume_file,
)
from vessl.service import serve_model
from vessl.util.constant import MODEL_SOURCE_EXPERIMENT, MODEL_SOURCE_LOCAL


def model_repository_status(latest_model: ResponseSimpleModelInfo) -> str:
    if latest_model:
        return latest_model.status
    return "Empty repository"


def model_repository_update_dt(
    latest_model: ResponseSimpleModelInfo, default: datetime
) -> datetime:
    if latest_model:
        return truncate_datetime(latest_model.created_dt)
    return truncate_datetime(default)


def model_repository_name_callback(ctx: click.Context, param: click.Parameter, value: str) -> str:
    if value:
        ctx.obj["repository"] = value
    return value


def model_repository_name_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    model_repositories = list_model_repositories()
    if len(model_repositories) == 0:
        raise click.UsageError(
            message="Create model repository with `vessl model-repository create`"
        )
    repository = prompt_choices("Model repository", [x.name for x in model_repositories])
    ctx.obj["repository"] = repository
    return repository


@click.command(name="model-repository", cls=VesslGroup)
def model_repository_cli():
    pass


@model_repository_cli.vessl_command()
@vessl_argument("name", type=click.STRING, required=True, prompter=model_repository_name_prompter)
@organization_name_option
def read(name: str):
    model_repository = read_model_repository(repository_name=name)
    print_data(
        {
            "ID": model_repository.id,
            "Name": model_repository.name,
            "Description": model_repository.description,
            "Status": model_repository_status(model_repository.model_summary.latest_model),
            "Organization": model_repository.organization.name,
            "Created": truncate_datetime(model_repository.created_dt),
            "Updated": truncate_datetime(model_repository.updated_dt),
        }
    )
    print_info(
        f"For more info: {format_url(Endpoint.model_repository.format(model_repository.organization.name, model_repository.name))}"
    )


@model_repository_cli.vessl_command()
@organization_name_option
def list():
    model_repositories = list_model_repositories()
    print_table(
        model_repositories,
        ["Name", "Status", "Models", "Created", "Updated"],
        lambda x: [
            x.name,
            model_repository_status(x.model_summary.latest_model),
            x.model_summary.total,
            truncate_datetime(x.created_dt),
            model_repository_update_dt(x.model_summary.latest_model, x.updated_dt),
        ],
    )


@model_repository_cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Model repository name"),
)
@click.option("-m", "--description", type=click.STRING)
@organization_name_option
def create(
    name: str,
    description: str,
):
    model_repository = create_model_repository(
        name=name,
        description=description,
    )
    print_success(
        f"Created '{model_repository.name}'.\n"
        f"For more info: {format_url(Endpoint.model_repository.format(model_repository.organization.name, model_repository.name))}"
    )


def model_number_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: int,
) -> int:
    repository = ctx.obj.get("repository")
    if repository is None:
        raise click.BadArgumentUsage(
            message="Argument `REPOSITORY_NAME` must be specified before `MODEL_NUMBER`.",
        )
    models = list_models(repository_name=repository)
    return prompt_choices("Model", [x.number for x in models])


@click.command(name="model", cls=VesslGroup)
def model_cli():
    pass


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
    callback=model_repository_name_callback,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@click.option(
    "--install-reqs",
    type=click.BOOL,
    is_flag=True,
    help="Install requirements before serving.",
)
@click.option(
    "--remote",
    type=click.BOOL,
    is_flag=True,
    hidden=True,
)
@organization_name_option
def serve(repository_name: str, model_number: int, install_reqs: bool, remote: bool):
    serve_model(repository_name, model_number, install_reqs, remote)


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
    callback=model_repository_name_callback,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@organization_name_option
def read(repository_name: str, model_number: int):
    model = read_model(repository_name=repository_name, model_number=model_number)
    experiment_numbers = []
    if model.experiment:
        experiment_numbers.append(model.experiment.number)

    metrics_summary = "None"
    if model.metrics_summary:
        metrics_keys = model.metrics_summary.latest.keys()
        metrics_summary = {}
        for key in metrics_keys:
            metrics_summary[key] = model.metrics_summary.latest[key].value

    print_data(
        {
            "ID": model.id,
            "Number": model.number,
            "Name": model.name,
            "Status": model.status,
            "Related experiments": experiment_numbers,
            "Creator": model.created_by.username,
            "Created": truncate_datetime(model.created_dt),
            "Metrics summary": metrics_summary,
        }
    )
    print_info(
        f"For more info: {format_url(Endpoint.model.format(model.model_repository.organization.name, model.model_repository.name, model.number))}"
    )


@model_cli.vessl_command()
@vessl_argument(
    "repository",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
)
@organization_name_option
def list(repository: str):
    models = list_models(repository_name=repository)
    print_table(
        models,
        ["Number", "Created", "Status"],
        lambda x: [x.number, truncate_datetime(x.created_dt), x.status],
    )


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@click.option("-p", "--path", type=click.Path(), default="", help="Defaults to root.")
@click.option("-r", "--recursive", is_flag=True)
@organization_name_option
def list_files(repository_name: str, model_number: int, path: str, recursive: bool):
    files = list_model_volume_files(
        repository_name=repository_name,
        model_number=model_number,
        need_download_url=False,
        path=path,
        recursive=recursive,
    )
    print_volume_files(files)


def model_source_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> str:
    return prompt_choices(
        "Source",
        [
            ("From an experiment", MODEL_SOURCE_EXPERIMENT),
            ("From local files", MODEL_SOURCE_LOCAL),
        ],
    )


def model_source_callback(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
):
    if value:
        ctx.obj["model_source"] = value
    return value


def experiment_id_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: int,
) -> int:
    model_source = ctx.obj.get("model_source")
    if model_source is None:
        raise click.BadOptionUsage(
            option_name="--source",
            message="Model source (`--source`) must be specified before experiment_id (`--experiment-id`).",
        )
    if model_source == MODEL_SOURCE_EXPERIMENT:
        experiments = list_experiments(statuses=["completed"])
        experiment = prompt_choices(
            "Experiment",
            [(f"{x.name} #{x.number}", x) for x in reversed(experiments)],
        )
        ctx.obj["experiment_number"] = experiment.number
        return experiment.id


def experiment_id_callback(
    ctx: click.Context,
    param: click.Parameter,
    value: int,
):
    if value and "experiment_number" not in ctx.obj:
        experiment = read_experiment_by_id(value)
        ctx.obj["experiment_number"] = experiment.number
    return value


def paths_prompter(
    ctx: click.Context,
    param: click.Parameter,
    value: List[str],
) -> List[str]:
    model_source = ctx.obj.get("model_source")
    if model_source is None:
        raise click.BadOptionUsage(
            option_name="--source",
            message="Model source (`--source`) must be specified before experiment_id (`--experiment-id`).",
        )
    paths = ["/"]
    if model_source == MODEL_SOURCE_EXPERIMENT:
        experiment_number = ctx.obj.get("experiment_number")
        if experiment_number is None:
            raise click.BadOptionUsage(
                option_name="--experiment-id",
                message="Experiment id (`--experiment-id`) must be specified before paths (`--paths`).",
            )
        files = list_experiment_output_files(
            experiment_number=experiment_number,
            need_download_url=False,
            recursive=True,
            worker_number=0,
        )
        if len(files) > 0:
            paths = prompt_checkbox(
                "Paths (Press -> to select and <- to unselect)",
                choices=[(f"{x.path} {format_size(x.size)}", x.path) for x in files],
            )
            if len(paths) == 0:
                paths = ["/"]

    return paths


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
)
@vessl_option("--model-name", type=click.STRING, help="Model name.")
@vessl_option(
    "--source",
    type=click.STRING,
    expose_value=False,
    prompter=model_source_prompter,
    callback=model_source_callback,
    help=f"{MODEL_SOURCE_EXPERIMENT} or {MODEL_SOURCE_LOCAL}.",
)
@vessl_option(
    "--experiment-id",
    type=click.INT,
    prompter=experiment_id_prompter,
    callback=experiment_id_callback,
    help=f"Experiment id to create model (only works for {MODEL_SOURCE_EXPERIMENT}).",
)
@vessl_option(
    "--path",
    type=click.STRING,
    multiple=True,
    prompter=paths_prompter,
    help=f"Path to create model (only works for {MODEL_SOURCE_EXPERIMENT}). Default: `/`",
)
@organization_name_option
@project_name_option
def create(
    repository_name: str,
    model_name: str = None,
    experiment_id: int = None,
    repository_description: str = None,
    path: str = None,
):
    model = create_model(
        repository_name=repository_name,
        repository_description=repository_description,
        experiment_id=experiment_id,
        model_name=model_name,
        paths=path,
    )
    print_success(
        f"Created '{model.model_repository.name}-{model.number}'.\n"
        f"For more info: {format_url(Endpoint.model.format(model.model_repository.organization.name, model.model_repository.name, model.number))}"
    )


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
    callback=model_repository_name_callback,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@vessl_argument(
    "source",
    type=click.Path(exists=True),
    required=True,
    prompter=generic_prompter("Source path"),
)
@vessl_argument(
    "dest",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Destination path", default="/"),
)
@organization_name_option
def upload(repository_name: str, model_number: int, source: str, dest: str):
    upload_model_volume_file(
        repository_name=repository_name,
        model_number=model_number,
        source_path=source,
        dest_path=dest,
    )
    print_success(f"Uploaded {source} to {dest}.")


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
    callback=model_repository_name_callback,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@vessl_argument(
    "source",
    type=click.Path(),
    required=True,
    prompter=generic_prompter("Source path", default="/"),
)
@vessl_argument(
    "dest",
    type=click.Path(),
    required=True,
    prompter=download_dest_path_prompter,
)
@organization_name_option
def download(repository_name: str, model_number: int, source: str, dest: str):
    download_model_volume_file(
        repository_name=repository_name,
        model_number=model_number,
        source_path=source,
        dest_path=dest,
    )
    print_success(f"Downloaded {source} to {dest}.")


@model_cli.vessl_command()
@vessl_argument(
    "repository_name",
    type=click.STRING,
    required=True,
    prompter=model_repository_name_prompter,
    callback=model_repository_name_callback,
)
@vessl_argument(
    "model_number",
    type=click.INT,
    required=True,
    prompter=model_number_prompter,
)
@vessl_argument("path", type=click.Path(), required=True, prompter=generic_prompter("File path"))
@organization_name_option
def delete_file(repository_name: str, model_number: int, path: str):
    delete_model_volume_file(
        repository_name=repository_name,
        model_number=model_number,
        path=path,
    )
    print_success(f"Deleted {path}.")



from typing import List

from kubernetes.config import KUBE_CONFIG_DEFAULT_LOCATION

from vessl.util.constant import VESSL_HELM_CHART_NAME
from vessl.util.exception import VesslRuntimeException


class ClusterInstallCommandBuilder:
    def __init__(self) -> None:
        self.default_arguments = [
            "helm",
            "install",
            "vessl",
            f"vessl/{VESSL_HELM_CHART_NAME}",
            "--create-namespace",
        ]
        self.namespace = "vessl"
        self.kubeconfig = KUBE_CONFIG_DEFAULT_LOCATION
        self.cluster_name = ""
        self.access_token = ""
        self.provider_type = "on-premise"
        self.local_path_provisioner_enabled = True

        # optional) As these overrides default values.yaml, should check if this is empty
        # and skip when empty
        self.helm_values = []
        self.agent_local_storage_class_name = ""
        self.pod_resource_path = ""

    def with_namespace(self, namespace: str) -> "ClusterInstallCommandBuilder":
        self.namespace = namespace
        return self

    def with_kubeconfig(self, kubeconfig_path: str) -> "ClusterInstallCommandBuilder":
        self.kubeconfig = kubeconfig_path
        return self

    def with_cluster_name(self, cluster_name: str) -> "ClusterInstallCommandBuilder":
        self.cluster_name = cluster_name
        return self

    def with_access_token(self, access_token: str) -> "ClusterInstallCommandBuilder":
        self.access_token = access_token
        return self

    def with_provider_type(self, provider_type: str) -> "ClusterInstallCommandBuilder":
        self.provider_type = provider_type
        return self

    def with_agent_local_storage_class_name(self, name: str) -> "ClusterInstallCommandBuilder":
        """
        - agent.localStorageClassName={name}
        """
        self.agent_local_storage_class_name = name
        return self

    def set_local_path_provisioner_enabled(self, enabled) -> "ClusterInstallCommandBuilder":
        """
        - local-path-provisioner.enabled={enabled}
        """
        self.local_path_provisioner_enabled = enabled
        return self

    def with_helm_values(self, helm_values: List[str]) -> "ClusterInstallCommandBuilder":
        self.helm_values = helm_values
        return self

    # @XXX(seokju) rough assumption: k0s user will install k0s nodes
    # if this becomes a problem, patch chart and cli.
    def set_pod_resource_path(self, path: str) -> "ClusterInstallCommandBuilder":
        """
        - dcgm-exporter.kubeletPath={path}
        """
        self.pod_resource_path = path
        return self

    def build(self) -> List[str]:
        if not self.cluster_name:
            raise VesslRuntimeException("cluster_name is required to build install command")
        if not self.access_token:
            raise VesslRuntimeException("access_token is required to build install command")
        self.default_arguments.extend(["--namespace", self.namespace])
        self.default_arguments.extend(["--kubeconfig", self.kubeconfig])
        self.default_arguments.extend(["--set", f"agent.clusterName={self.cluster_name}"])
        self.default_arguments.extend(["--set", f"agent.accessToken={self.access_token}"])
        self.default_arguments.extend(["--set", f"agent.providerType={self.provider_type}"])
        self.default_arguments.extend(
            [
                "--set",
                f"local-path-provisioner.enabled={'true' if self.local_path_provisioner_enabled else 'false'}",
            ]
        )

        # check optionals
        if self.agent_local_storage_class_name:
            self.default_arguments.extend(
                [
                    "--set",
                    f"agent.localStorageClassName={self.agent_local_storage_class_name}",
                ]
            )
        for helm_value in self.helm_values:
            if helm_value:
                self.default_arguments.extend(["--set", helm_value])

        if self.pod_resource_path:
            self.default_arguments.extend(
                [
                    "--set",
                    f"dcgm-exporter.kubeletPath={self.pod_resource_path}",
                ]
            )
        return self.default_arguments
